


#include "Camera/KOFDefaultCamera.h"

#include "DrawDebugHelpers.h"
#include "Camera/CameraComponent.h"
#include "Character/KOFBaseCharacter.h"
#include "GameModes/KOFDefaultGameMode.h"
#include "Kismet/GameplayStatics.h"

AKOFDefaultCamera::AKOFDefaultCamera()
{
	PrimaryActorTick.bCanEverTick = true;
	
	MinFOV = 80.0f;
	MaxFOV = 85.0f;
	GetCameraComponent()->SetFieldOfView(MinFOV);

	HeightOffset = 60.0f;
	ZAveragingFactor = 2.0f;

	MovementYDistanceThreshold = 500.0f;
}

void AKOFDefaultCamera::Tick(float DeltaSeconds)
{
	Super::Tick(DeltaSeconds);

	// NOTE(kevin): This will only return the game mode if we are the server. Since this game is local only we should always
	// be both server and client
	AKOFDefaultGameMode* GameMode = Cast<AKOFDefaultGameMode>(GetWorld()->GetAuthGameMode());

	if(GameMode)
	{
		AKOFBaseCharacter* P1 = GameMode->GetP1CurrentCharacter();
		AKOFBaseCharacter* P2 = GameMode->GetP2CurrentCharacter();
		
		check(P1);
		check(P2);

		FVector P1Loc = P1->GetActorLocation();
		FVector P2Loc = P2->GetActorLocation();
		FVector CameraLoc = GetActorLocation();

		float HalfStageLength = GameMode->GetStageLength() / 2.0f;
		FVector MidPoint = FVector(CameraLoc.X, FMath::Clamp((P1Loc.Y + P2Loc.Y) / 2.0f, -HalfStageLength, HalfStageLength), ((P1Loc.Z + P2Loc.Z) / ZAveragingFactor) + HeightOffset);
		SetActorLocation(MidPoint);
		
		// set FOV
	}
}

void AKOFDefaultCamera::UpdateCamera()
{
	// NOTE(kevin): This will only return the game mode if we are the server. Since this game is local only we should always
	// be both server and client
	AKOFDefaultGameMode* GameMode = Cast<AKOFDefaultGameMode>(GetWorld()->GetAuthGameMode());

	if(GameMode)
	{
		AKOFBaseCharacter* P1 = GameMode->GetP1CurrentCharacter();
		AKOFBaseCharacter* P2 = GameMode->GetP2CurrentCharacter();
		
		FVector P1Loc = P1->GetActorLocation();
		FVector P2Loc = P2->GetActorLocation();
		FVector CameraLoc = GetActorLocation();
	
		float HalfMoveYThreshold = MovementYDistanceThreshold / 2.0f;

		float YDistance = FMath::Abs(P1Loc.Y - P2Loc.Y);

		/**
		 *	If one of the characters starts to move far enough to either side of the camera (determined by Movement Y distance threshold),
		 *	then start pushing the camera in that same direction
		 */
		/*
		if((P1Loc.Y < CameraLoc.Y - HalfMoveYThreshold || P2Loc.Y < CameraLoc.Y - HalfMoveYThreshold) ||
		(P1Loc.Y > CameraLoc.Y + HalfMoveYThreshold || P2Loc.Y > CameraLoc.Y + HalfMoveYThreshold))
		{
		//FVector MidPoint = FVector(CameraLoc.X, (P1Loc.Y + P2Loc.Y) / 2.0f, ((P1Loc.Z + P2Loc.Z) / ZAveragingFactor) + HeightOffset);
		FVector MidPoint = FVector(CameraLoc.X, (P1Loc.Y + P2Loc.Y) / 2.0f, CameraLoc.Z);
		SetActorLocation(MidPoint);
		}
		*/

		
		// leftest = MaxF(float32(Min(s.stage.p[0].startx, s.stage.p[1].startx))*s.stage.localscl,-(float32(s.gameWidth)/2)/s.cam.BaseScale()+s.screenleft) - ox
		// rightest = MinF(float32(Max(s.stage.p[0].startx, s.stage.p[1].startx))*s.stage.localscl, (float32(s.gameWidth)/2)/s.cam.BaseScale()-s.screenright) - ox
	
		float MoveRightVal = P1Loc.Y > CameraLoc.Y + HalfMoveYThreshold ? P1Loc.Y - (CameraLoc.Y + HalfMoveYThreshold) : 0.0f;
		float MoveLeftVal = P1Loc.Y < CameraLoc.Y - HalfMoveYThreshold ? P1Loc.Y - (CameraLoc.Y - HalfMoveYThreshold) : 0.0f;
	
		FVector NewLocation = FVector(CameraLoc.X, (CameraLoc.Y + MoveRightVal - MoveLeftVal), CameraLoc.Z);
	}
}


